		`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Companppmy: 
// 			Engineer: 
// 
// Create Date:    09:39:52 05/13/2011 
// Design Name: 
// Module Name:    ddscontrol 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Top level file for Minikits DDS controller
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module ddscontrol(
	input wire clk100,
	input wire ser_rx,
	output wire ser_tx,
	
	output reg [7:0] pmod
    );


	// Picoblaze control signals
	wire	[11:0]	address;
	wire	[17:0]	instruction;
	wire			bram_enable;
	wire	[7:0]		port_id;
	wire	[7:0]		out_port;
	reg	[7:0]		in_port;
	wire			write_strobe;
	wire			k_write_strobe;
	wire			read_strobe;
	reg			interrupt = 0;
	wire			interrupt_ack;
	wire			kcpsm6_sleep;        
	wire			kcpsm6_reset;
	wire			cpu_reset;
	wire			rdl;
	wire			int_request;
	
	kcpsm6 #(
		.interrupt_vector	(12'h3FF),
		.scratch_pad_memory_size(64),
		.hwbuild		(8'h00))
	processor (
		.address 		(address),
		.instruction 	(instruction),
		.bram_enable 	(bram_enable),
		.port_id 		(port_id),
		.write_strobe 	(write_strobe),
		.k_write_strobe 	(k_write_strobe),
		.out_port 		(out_port),
		.read_strobe 	(read_strobe),
		.in_port 		(in_port),
		.interrupt 		(interrupt),
		.interrupt_ack 	(interrupt_ack),
		.reset 		(kcpsm6_reset),
		.sleep		(kcpsm6_sleep),
		.clk 			(clk100)); 

	assign kcpsm6_sleep = 1'b0;
	//assign kcpsm6_reset = 1'b0;
	
	ddscontrol_pb #(
		.C_FAMILY		   ("S6"),   	//Family 'S6' or 'V6'
		.C_RAM_SIZE_KWORDS	(2),  	//Program size '1', '2' or '4'
		.C_JTAG_LOADER_ENABLE	(1))  	//Include JTAG Loader when set to '1' 
	program_rom (    				//Name to match your PSM file
		.rdl 			(kcpsm6_reset),
		.enable 		(bram_enable),
		.address 		(address),
		.instruction 	(instruction),
		.clk 			(clk100));

	// UART
		
	reg [9:0] 	baud_count;
	reg  		en_16_x_baud;
	wire  	write_to_uart;
	wire  	tx_full;
	wire  	tx_half_full;
	reg  		read_from_uart;
	wire [7:0] 	rx_data;
	wire  	rx_data_present;
	wire  	rx_full;
	wire  	rx_half_full;
	wire [7:0] uart_status_port;
	
	assign uart_status_port = {3'b000,rx_data_present,rx_full,rx_half_full,tx_full,tx_half_full};

	uart_tx transmit (
		.data_in(out_port),
		.write_buffer(write_to_uart),
		.reset_buffer(1'b0),
		.en_16_x_baud(en_16_x_baud),
		.serial_out(ser_tx),
		.buffer_full(tx_full),
		.buffer_half_full(tx_half_full),
		.clk(clk100));

	uart_rx receive (
		.serial_in(ser_rx),
		.data_out(rx_data),
		.read_buffer(read_from_uart),
		.reset_buffer(1'b0),
		.en_16_x_baud(en_16_x_baud),
		.buffer_data_present(rx_data_present),
		.buffer_full(rx_full),
		.buffer_half_full(rx_half_full),
		.clk(clk100));

	// Picoblaze input ports

	always @(posedge clk100) begin
		case(port_id[1:0])
			2'b00: in_port <= uart_status_port;
			2'b01: in_port <= rx_data;
			default: in_port <= 8'bXXXXXXXX;
		endcase

		// Read strobe for UART receiver FIFO buffer.
		read_from_uart <= read_strobe && (port_id[1:0] == 2'b01);

	end
	
	// Picoblaze output ports
	
	always @(posedge clk100) begin
		if(write_strobe)
			case(port_id[1:0])
			2'b00: pmod <= out_port;
			
			endcase
	end

	// Write to UART transmitter FIFO buffer at address 01 hex.
	assign write_to_uart = write_strobe && (port_id[1:0] == 2'b01);
	
	// UART baud rate generator - 9600 baud
	always @(posedge clk100)
		if (baud_count == 651) begin
			baud_count <= 1'b0;
			en_16_x_baud <= 1'b1;
		end else begin
			baud_count <= baud_count + 1'b1;
			en_16_x_baud <= 1'b0;
		end
	
endmodule
